import os
import sys
import zmq
# from uci import Uci
import json
# from rrd_manager import rrdtool_update, directory_exists, create_rrd_file
import argparse
# from luci_uci import add_gauge_for_uci, add_gauges_to_uci
from pynovatec_rrd import RRDGaugeManager

parser = argparse.ArgumentParser()
parser.add_argument("-p", "--port", type=int, default=4447, help="Port to listen on")
parser.add_argument("-ip", "--ip", type=str, default="127.0.0.1", help="IP to listen on")
parser.add_argument("-dir", "--directory_path", type=str, default="/tmp/rrd/OpenWrt", help="Directory to save rrd files in")
parser.add_argument("-machine_type", "--machine_type", type=str, default="gauges", help="What type of machine is this? pump, power, etc.")
args = parser.parse_args()

# gauges = []

# Socket to talk to server
context = zmq.Context()
socket = context.socket(zmq.SUB)
socket.connect ("tcp://%s:%s" % (args.ip, args.port))
socket.subscribe("")

# u = Uci()
p_type = "gauges"
default_hostname = 'OpenWrt'
# mapper = {"PH1": "Phase 1", "PH2": "Phase 2", "PH3": "Phase 3"}

print("Reporting data from server from port %s..." % args.port)

def process_message(input_json):
    s=string.split("~~")
    message = json.loads(s[3])
    print("Message: ", json.dumps(message,indent=3))

    if "header" not in message:
        return
    header = message['header']
    print("Header: ", header)
    output_format = int(float(header.get("output_format",1)))
    if output_format == 2:
        output_format_2(message)
    else:
        output_format_1(message)
        
def output_format_2(message):
    global rrd_manager

    print("OUTPUT FORMAT 2")
    timestamp = message["header"]["timestamp"]
    machine_name = message["header"].get("machineName")
    machine_id = message["header"].get("machineId")

    gauges = message["gauges"]
    
    for subassembly_name, subassembly_gauges in gauges.items():
        for gauge_name, gauge_info in subassembly_gauges.items():
            if "kpis" in gauge_info:
                data = create_data(gauge_info["kpis"],timestamp,subassembly_name,gauge_name, machine_name, machine_id)
                data.update({'machine_id': machine_id})
                # save_to_rrd(data)
                rrd_manager.save(data)
        # add_gauges_to_uci("gauges",subassembly_name)

exclude_names = ["trendYMax","trendYMin","red_threshold","yellow_threshold"]

def get_kpis(mart):
    kpis = {}
    for name,value in mart.items():
        if name in exclude_names:
            continue
        kpis[name] = value
    return kpis

# Output_format_1 is very kludige, we need to exclude some mart values.

def output_format_1(message):
    global rrd_manager
    print("OUTPUT FORMAT 1")
    timestamp = message["header"]["timestamp"]
    machine_name = message["header"].get("machineName")
    machine_id = message["header"].get("machineId")
    if "gauge" in message:
        for key,gauge in message["gauge"].items(): #for key,value in gauges.items()
            print("******* Gauge: ", key)
            kpis = None
            if gauge["success"] == False or gauge["success"] == "false":
                continue
            s = key.split(":")
            subassembly_instance=s[0].strip()
            gauge_name = s[1].strip()

            if "type" in gauge.keys():
                gtype = gauge["type"]
            elif "gaugeType" in gauge.keys():
                gtype = gauge["gaugeType"]
            else:
                continue
            print("gtype: ",gtype)
            if gtype == "bar":
                if 'mart' in gauge:
                    kpis = get_kpis(gauge["mart"])
                else:
                    kpis = gauge["value"]
            elif (gtype == "widget" or gtype == "color" or gtype == "value"):
                if 'mart' not in gauge:
                    continue
                kpis = get_kpis(gauge["mart"])
            if kpis:
                data = create_data(kpis,timestamp,subassembly_instance,gauge_name, machine_name, machine_id)
                data.update({'machine_id': machine_id})
                # save_to_rrd(data)
                rrd_manager.save(data)
            # add_gauges_to_uci("gauges",subassembly_instance)

def create_data(v, timestamp, subassembly, gauge_name, machine_name, machine_id):
    # print("KEY: ",key)
    if machine_name is None:
        machine_name = machine_id
    step=1200
    if gauge_name in ["VRMS","IRMS","Power Factor","Active Power"]:
        step=120
    if isinstance(v,dict):
        print("Is dict")
        values = list(v.values())
    else:
        values = v
    data = {
        "category_type": "gauges",
        "category": machine_name,
        "subcategory": subassembly,
        "rrd_name": gauge_name,
        "values": values,
        "length": len(values),
        "timestamp": timestamp,
        "gauge_name": gauge_name,
        "subassembly": subassembly,
        "step": step
    }
    return data

# def save_to_rrd(message):
#     # try:
#     #     global p_type
#     #     hostname = u.get('luci_statistics', 'collectd', 'Hostname') + "/"
#     # except:
#     #     hostname = default_hostname + "/"
#     # rrd_dir = u.get('luci_statistics', 'collectd_rrdtool', 'DataDir') + "/"
#     print("Save to rrd: ", message)
#     # key = message["name"].split(":")
#     # add_gauge_for_uci(message['subcategory'],message['rrd_name'])
#     add_gauge_for_uci(message)

#     save_dir = args.directory_path + "/"+args.machine_type+"-"+message.get("subcategory")+"/"
#     filename = save_dir + message['rrd_name'].replace(" ", "")+'.rrd'

#     print("save_dir: ",save_dir)
#     print("filename: ",filename)
#     if not os.path.exists(filename):
#         create_rrd_file(save_dir, filename, message["length"], message["step"], starttime=message["timestamp"]-1)

#     try:
#         rrdtool_update(filename,message['rrd_name'], str(message["timestamp"]), message["values"])
#     except Exception as e:
#         print(f"Error updating file: {str(e)}")

rrd_manager = RRDGaugeManager(args.directory_path,args.machine_type)

while True:
  string = socket.recv_string()
  if len(string) == 0:
        print("No msg: ",len(string))
        continue
  else:
    if "output" in str(string):
        print(f"RECV: {str(string)}")
        process_message(str(string))
