import rrdtool
import os
from .luci_uci import add_gauge_for_uci


def directory_exists(directory_path):
    return os.path.exists(directory_path) and os.path.isdir(directory_path)

class RRDManager:
    def __init__(self,rrd_dir=None, machine_type=None):
        self.rrd_dir = rrd_dir
        self.machine_type = machine_type

    def save(self,message):
        message['rrd_name'] = message['rrd_name'].replace("(", "_").replace(")", "").replace(" ", "")
        if "gauge_name" in message:
            message['gauge_name'] = message['gauge_name'].replace("(", "_").replace(")", "")
        message['category'] = message['category'].replace(" ", "_")
        # try:
        #     global p_type
        #     hostname = u.get('luci_statistics', 'collectd', 'Hostname') + "/"
        # except:
        #     hostname = default_hostname + "/"
        # rrd_dir = u.get('luci_statistics', 'collectd_rrdtool', 'DataDir') + "/"
        print("Save to rrd: ", message)
        # key = message["name"].split(":")
        # add_gauge_for_uci(message['subcategory'],message['rrd_name'])
        add_gauge_for_uci(message)

        # save_dir = args.directory_path + "/"+args.machine_type+"-"+message.get("subcategory")+"/"
        machine_type = message['category']
        save_dir = self.rrd_dir + "/"+machine_type+"-"+message.get("subcategory")+"/"

        filename = save_dir + message['rrd_name']+'.rrd'

        print("save_dir: ",save_dir)
        print("filename: ",filename)

        if not os.path.exists(filename):
            # self.rrd_create(save_dir, filename, message["length"], machine_type,message["step"], starttime=message["timestamp"]-1)
            self.rrd_create(save_dir, filename, message["length"], machine_type,message["step"], starttime=message["timestamp"])

        try:
            self.rrdtool_update(filename, message["timestamp"], message["values"])
        except Exception as e:
            print(f"Error updating file: {str(e)}")

    def rrdtool_update(self,file, timestamp, values):

        ts = "N" if timestamp is None else str(timestamp)
        value_list = ":".join(map(str, values))
        rrdtool.update(file, ts+':'+value_list)
        print(f"Updated RRD file: {file} --- with {timestamp}:{value_list}")

class RRDGaugeManager(RRDManager):
    def __init__(self,*args,**kwargs):
        super().__init__(*args,**kwargs)

    def rrd_create(self,save_dir, filename, length, machine_type,step, starttime=None):
        print("Save dir: ",save_dir)
        if not directory_exists(save_dir):
            try:
                os.makedirs(save_dir)
            except:
                return

        rra_configs = [
            {'cf': 'AVERAGE', 'rows': 288, 'pdp_per_row': 1, 'xff': 0.1},
            {'cf': 'AVERAGE', 'rows': 288, 'pdp_per_row': 10, 'xff': 0.1},
            {'cf': 'AVERAGE', 'rows': 288, 'pdp_per_row': 60, 'xff': 0.1},
            {'cf': 'AVERAGE', 'rows': 288, 'pdp_per_row': 360, 'xff': 0.1}
        ]

        ds_configs = []
        for i in range(1, length + 1):
            ds_name = f'value_{i}'
            ds_config = f'DS:{ds_name}:GAUGE:{step}:-100:10000'
            ds_configs.append(ds_config)

        rra_configs = [
            f'RRA:{config["cf"]}:{config["xff"]}:{config["pdp_per_row"]}:{config["rows"]}'
            for config in rra_configs
        ]

        rrdargs = [filename,
        f'--step={int(step/2)}',
        '--no-overwrite'
        ] 
        if starttime is not None:
            rrdargs.append(f'--start={starttime - 1}')

        rrdtool.create(*rrdargs,*ds_configs,*rra_configs)
        print(f'RRD file "{filename}" created successfully.')


# rrd_dir = None
# machine_type = None

# def init_rrd(sd,mt):
#     global rrd_dir
#     global machine_type

#     rrd_dir = sd
#     machine_type = mt

# def save_to_rrd(message):
#     global rrd_dir
#     global machine_type

#     # try:
#     #     global p_type
#     #     hostname = u.get('luci_statistics', 'collectd', 'Hostname') + "/"
#     # except:
#     #     hostname = default_hostname + "/"
#     # rrd_dir = u.get('luci_statistics', 'collectd_rrdtool', 'DataDir') + "/"
#     print("Save to rrd: ", message)
#     # key = message["name"].split(":")
#     # add_gauge_for_uci(message['subcategory'],message['rrd_name'])
#     add_gauge_for_uci(message)

#     # save_dir = args.directory_path + "/"+args.machine_type+"-"+message.get("subcategory")+"/"

#     save_dir = rrd_dir + "/"+machine_type+"-"+message.get("subcategory")+"/"

#     filename = save_dir + message['rrd_name'].replace(" ", "")+'.rrd'

#     print("save_dir: ",save_dir)
#     print("filename: ",filename)
#     if not os.path.exists(filename):
#         create_rrd_file(save_dir, filename, message["length"], message["step"], starttime=message["timestamp"]-1)

#     try:
#         rrdtool_update(filename,message['rrd_name'], str(message["timestamp"]), message["values"])
#     except Exception as e:
#         print(f"Error updating file: {str(e)}")



# def rrdtool_update(file,gauge_name, timestamp, values):

#     value_list = ":".join(map(str, values))
#     rrdtool.update(file, timestamp+':'+value_list)
#     print(f"Updated RRD file: {file} --- with {timestamp}:{value_list}")


# def create_rrd_file(save_dir, filename, length, step, starttime=None):
#     print("Save dir: ",save_dir)
#     if not directory_exists(save_dir):
#       try:
#           os.makedirs(save_dir)
#       except:
#           return

#     rra_configs = [
#         {'cf': 'AVERAGE', 'rows': 288, 'pdp_per_row': 1, 'xff': 0.1},
#         {'cf': 'AVERAGE', 'rows': 288, 'pdp_per_row': 10, 'xff': 0.1},
#         {'cf': 'AVERAGE', 'rows': 288, 'pdp_per_row': 60, 'xff': 0.1},
#         {'cf': 'AVERAGE', 'rows': 288, 'pdp_per_row': 360, 'xff': 0.1}
#     ]

#     ds_configs = []
#     for i in range(1, length + 1):
#         ds_name = f'value_{i}'
#         ds_config = f'DS:{ds_name}:GAUGE:{step}:-100:10000'
#         ds_configs.append(ds_config)

#     rra_configs = [
#         f'RRA:{config["cf"]}:{config["xff"]}:{config["pdp_per_row"]}:{config["rows"]}'
#         for config in rra_configs
#     ]

#     rrdargs = [filename,
#        f'--step={int(step/2)}',
#        '--no-overwrite'
#     ] 
#     if starttime is not None:
#         rrdargs.append(f'--start={starttime}')

#     rrdtool.create(*rrdargs,*ds_configs,*rra_configs)
#     print(f'RRD file "{filename}" created successfully.')